@@ -235,7 +235,16 @@ object NextTurnAutomation {
                     val valueOfOne = evaluation.evaluateSellCost(TradeOffer(ourGold.name, ourGold.type, 1, ourGold.duration), civInfo, otherCiv)
                     val amountToGive = min(deltaInOurFavor / valueOfOne, ourGold.amount)
                     deltaInOurFavor -= amountToGive * valueOfOne
-                    counterofferGifts.add(TradeOffer(ourGold.name, ourGold.type, amountToGive, ourGold.duration))
+                    if (amountToGive > 0) {
+                        counterofferGifts.add(
+                            TradeOffer(
+                                ourGold.name,
+                                ourGold.type,
+                                amountToGive,
+                                ourGold.duration
+                            )
+                        )
+                    }
                 }
             }
         }
@@ -281,38 +290,44 @@ object NextTurnAutomation {
         return
     }
 
-    /** allow AI to spend money to purchase city-state friendship, buildings & unit */
-    private fun useGold(civInfo: Civilization) {
-        if (civInfo.getHappiness() > 0 && civInfo.hasUnique(UniqueType.CityStateCanBeBoughtForGold)) {
-            for (cityState in civInfo.getKnownCivs().filter { it.isCityState() } ) {
-                if (cityState.cityStateFunctions.canBeMarriedBy(civInfo))
-                    cityState.cityStateFunctions.diplomaticMarriage(civInfo)
-                if (civInfo.getHappiness() <= 0) break // Stop marrying if happiness is getting too low
+    private fun useGoldForCityStates(civ: Civilization) {
+        val knownCityStates = civ.getKnownCivs().filter { it.isCityState() }
+
+        // canBeMarriedBy checks actual cost, but it can't be below 500*speedmodifier, and the later check is expensive
+        if (civ.gold >= 330 && civ.getHappiness() > 0 && civ.hasUnique(UniqueType.CityStateCanBeBoughtForGold)) {
+            for (cityState in knownCityStates.toList() ) {  // Materialize sequence as diplomaticMarriage may kill a CS
+                if (cityState.cityStateFunctions.canBeMarriedBy(civ))
+                    cityState.cityStateFunctions.diplomaticMarriage(civ)
+                if (civ.getHappiness() <= 0) break // Stop marrying if happiness is getting too low
             }
         }
 
-        if (civInfo.wantsToFocusOn(Victory.Focus.Culture)) {
-            for (cityState in civInfo.getKnownCivs()
-                    .filter { it.isCityState() && it.cityStateFunctions.canProvideStat(Stat.Culture) }) {
-                val diploManager = cityState.getDiplomacyManager(civInfo)
+        if (civ.gold < 250) return  // skip checks if tryGainInfluence will bail anyway
+        if (civ.wantsToFocusOn(Victory.Focus.Culture)) {
+            for (cityState in knownCityStates.filter { it.cityStateFunctions.canProvideStat(Stat.Culture) }) {
+                val diploManager = cityState.getDiplomacyManager(civ)
                 if (diploManager.getInfluence() < 40) { // we want to gain influence with them
-                    tryGainInfluence(civInfo, cityState)
+                    tryGainInfluence(civ, cityState)
                 }
             }
         }
 
-        if (!civInfo.isCityState()) {
-            val potentialAllies = civInfo.getKnownCivs().filter { it.isCityState() }
-            if (potentialAllies.any()) {
-                val cityState =
-                    potentialAllies.maxByOrNull { valueCityStateAlliance(civInfo, it) }!!
-                if (cityState.getAllyCiv() != civInfo.civName && valueCityStateAlliance(civInfo, cityState) > 0) {
-                    tryGainInfluence(civInfo, cityState)
-                }
-            }
+        if (civ.gold < 250 || knownCityStates.none()) return
+        val cityState = knownCityStates
+            .filter { it.getAllyCiv() != civ.civName }
+            .associateWith { valueCityStateAlliance(civ, it) }
+            .maxByOrNull { it.value }?.takeIf { it.value > 0 }?.key
+        if (cityState != null) {
+            tryGainInfluence(civ, cityState)
         }
+    }
 
-        for (city in civInfo.cities.sortedByDescending { it.population.population }) {
+    /** allow AI to spend money to purchase city-state friendship, buildings & unit */
+    private fun useGold(civ: Civilization) {
+        if (civ.isMajorCiv())
+            useGoldForCityStates(civ)
+
+        for (city in civ.cities.sortedByDescending { it.population.population }) {
             val construction = city.cityConstructions.getCurrentConstruction()
             if (construction is PerpetualConstruction) continue
             if ((construction as INonPerpetualConstruction).canBePurchasedWithStat(city, Stat.Gold)
@@ -321,12 +336,18 @@ object NextTurnAutomation {
             }
         }
 
-        maybeBuyCityTiles(civInfo)
+        maybeBuyCityTiles(civ)
     }
 
     private fun maybeBuyCityTiles(civInfo: Civilization) {
         if (civInfo.gold <= 0)
             return
+        // Don't buy tiles in the very early game. It is unlikely that we already have the required
+        // tech, the necessary worker and that there is a reasonable threat from another player to
+        // grab the tile. We could also check all that, but it would require a lot of cycles each
+        // turn and this is probably a good approximation.
+        if (civInfo.gameInfo.turns < (civInfo.gameInfo.speed.scienceCostModifier * 20).toInt())
+            return
 
         val highlyDesirableTiles: SortedMap<Tile, MutableSet<City>> = TreeMap(
             compareByDescending<Tile?> { it?.naturalWonder != null }
@@ -347,7 +368,8 @@ object NextTurnAutomation {
                         (it.hasViewableResource(civInfo)
                                 && it.tileResource.resourceType == ResourceType.Strategic &&
                                 (civInfo.getCivResourcesByName()[it.resource!!] ?: 0) <= 3)
-                it.isVisible(civInfo) && it.getOwner() == null &&
+                it.isVisible(civInfo) && it.getOwner() == null
+                        && it.neighbors.any { neighbor -> neighbor.getCity() == city }
                         (hasNaturalWonder || hasLuxuryCivDoesntOwn || hasResourceCivHasNoneOrLittle)
             }
             for (highlyDesirableTileInCity in highlyDesirableTilesInCity) {
@@ -572,7 +594,7 @@ object NextTurnAutomation {
      *  a unit and selling a building to make room. Can happen due to trades etc */
     private fun freeUpSpaceResources(civInfo: Civilization) {
         // No need to build spaceship parts just yet
-        if (civInfo.gameInfo.ruleset.victories.none { civInfo.victoryManager.getNextMilestone(it.key)?.type == MilestoneType.AddedSSPartsInCapital } )
+        if (civInfo.gameInfo.ruleset.victories.none { civInfo.victoryManager.getNextMilestone(it.value)?.type == MilestoneType.AddedSSPartsInCapital } )
             return
 
         for (resource in civInfo.gameInfo.spaceResources) {
@@ -630,12 +652,16 @@ object NextTurnAutomation {
         if (civInfo.religionManager.religionState != ReligionState.FoundingReligion) return
         val availableReligionIcons = civInfo.gameInfo.ruleset.religions
             .filterNot { civInfo.gameInfo.religions.values.map { religion -> religion.name }.contains(it) }
+        val favoredReligion = civInfo.nation.favoredReligion
         val religionIcon =
-            if (civInfo.nation.favoredReligion in availableReligionIcons) civInfo.nation.favoredReligion
+            if (favoredReligion != null && favoredReligion in availableReligionIcons) favoredReligion
             else availableReligionIcons.randomOrNull()
                 ?: return // Wait what? How did we pass the checking when using a great prophet but not this?
+
+        civInfo.religionManager.foundReligion(religionIcon, religionIcon)
+
         val chosenBeliefs = chooseBeliefs(civInfo, civInfo.religionManager.getBeliefsToChooseAtFounding()).toList()
-        civInfo.religionManager.chooseBeliefs(chosenBeliefs, religionIcon, religionIcon)
+        civInfo.religionManager.chooseBeliefs(chosenBeliefs)
     }
 
     private fun enhanceReligion(civInfo: Civilization) {
@@ -795,15 +821,18 @@ object NextTurnAutomation {
 
         if (enemyCivs.none()) return
 
+        val minMotivationToAttack = 20
         val civWithBestMotivationToAttack = enemyCivs
-                .map { Pair(it, motivationToAttack(civInfo, it)) }
+                .map { Pair(it, hasAtLeastMotivationToAttack(civInfo, it, minMotivationToAttack)) }
                 .maxByOrNull { it.second }!!
 
-        if (civWithBestMotivationToAttack.second >= 20)
+        if (civWithBestMotivationToAttack.second >= minMotivationToAttack)
             civInfo.getDiplomacyManager(civWithBestMotivationToAttack.first).declareWar()
     }
 
-    private fun motivationToAttack(civInfo: Civilization, otherCiv: Civilization): Int {
+    /** Will return the motivation to attack, but might short circuit if the value is guaranteed to
+     * be lower than `atLeast`. So any values below `atLeast` should not be used for comparison. */
+    private fun hasAtLeastMotivationToAttack(civInfo: Civilization, otherCiv: Civilization, atLeast: Int): Int {
         val closestCities = getClosestCities(civInfo, otherCiv) ?: return 0
         val baseForce = 30f
 
@@ -837,12 +866,6 @@ object NextTurnAutomation {
                     && (owner == otherCiv || owner == null || civInfo.diplomacyFunctions.canPassThroughTiles(owner))
         }
 
-        val reachableEnemyCitiesBfs = BFS(civInfo.getCapital()!!.getCenterTile()) { isTileCanMoveThrough(it) }
-        reachableEnemyCitiesBfs.stepToEnd()
-        val reachableEnemyCities = otherCiv.cities.filter { reachableEnemyCitiesBfs.hasReachedTile(it.getCenterTile()) }
-        if (reachableEnemyCities.isEmpty()) return 0 // Can't even reach the enemy city, no point in war.
-
-
         val modifierMap = HashMap<String, Int>()
         val combatStrengthRatio = ourCombatStrength / theirCombatStrength
         val combatStrengthModifier = when {
@@ -858,14 +881,6 @@ object NextTurnAutomation {
         if (closestCities.aerialDistance > 7)
             modifierMap["Far away cities"] = -10
 
-        val landPathBFS = BFS(ourCity.getCenterTile()) {
-            it.isLand && isTileCanMoveThrough(it)
-        }
-
-        landPathBFS.stepUntilDestination(theirCity.getCenterTile())
-        if (!landPathBFS.hasReachedTile(theirCity.getCenterTile()))
-            modifierMap["No land path"] = -10
-
         val diplomacyManager = civInfo.getDiplomacyManager(otherCiv)
         if (diplomacyManager.hasFlag(DiplomacyFlags.ResearchAgreement))
             modifierMap["Research Agreement"] = -5
@@ -902,7 +917,35 @@ object NextTurnAutomation {
                 modifierMap["About to win"] = 15
         }
 
-        return modifierMap.values.sum()
+        var motivationSoFar = modifierMap.values.sum()
+
+        // We don't need to execute the expensive BFSs below if we're below the threshold here
+        // anyways, since it won't get better from those, only worse.
+        if (motivationSoFar < atLeast) {
+            return motivationSoFar
+        }
+
+
+        val landPathBFS = BFS(ourCity.getCenterTile()) {
+            it.isLand && isTileCanMoveThrough(it)
+        }
+
+        landPathBFS.stepUntilDestination(theirCity.getCenterTile())
+        if (!landPathBFS.hasReachedTile(theirCity.getCenterTile()))
+            motivationSoFar -= -10
+
+        // We don't need to execute the expensive BFSs below if we're below the threshold here
+        // anyways, since it won't get better from those, only worse.
+        if (motivationSoFar < atLeast) {
+            return motivationSoFar
+        }
+
+        val reachableEnemyCitiesBfs = BFS(civInfo.getCapital()!!.getCenterTile()) { isTileCanMoveThrough(it) }
+        reachableEnemyCitiesBfs.stepToEnd()
+        val reachableEnemyCities = otherCiv.cities.filter { reachableEnemyCitiesBfs.hasReachedTile(it.getCenterTile()) }
+        if (reachableEnemyCities.isEmpty()) return 0 // Can't even reach the enemy city, no point in war.
+
+        return motivationSoFar
     }
 
 
@@ -919,8 +962,10 @@ object NextTurnAutomation {
                 .filter { it.tradeRequests.none { tradeRequest -> tradeRequest.requestingCiv == civInfo.civName && tradeRequest.trade.isPeaceTreaty() } }
 
         for (enemy in enemiesCiv) {
-            val motivationToAttack = motivationToAttack(civInfo, enemy)
-            if (motivationToAttack >= 10) continue // We can still fight. Refuse peace.
+            if(hasAtLeastMotivationToAttack(civInfo, enemy, 10) >= 10) {
+                // We can still fight. Refuse peace.
+                continue
+            }
 
             // pay for peace
             val tradeLogic = TradeLogic(civInfo, enemy)
@@ -962,15 +1007,23 @@ object NextTurnAutomation {
     }
 
     private fun automateCities(civInfo: Civilization) {
+        val ownMilitaryStrength = civInfo.getStatForRanking(RankingType.Force)
+        val sumOfEnemiesMilitaryStrength =
+                civInfo.gameInfo.civilizations
+                    .filter { it != civInfo && !it.isBarbarian() && civInfo.isAtWarWith(it) }
+                    .sumOf { it.getStatForRanking(RankingType.Force) }
+        val civHasSignificantlyWeakerMilitaryThanEnemies =
+                ownMilitaryStrength < sumOfEnemiesMilitaryStrength * 0.66f
         for (city in civInfo.cities) {
             if (city.isPuppet && city.population.population > 9
-                    && !city.isInResistance()) {
+                    && !city.isInResistance()
+            ) {
                 city.annexCity()
             }
 
             city.reassignAllPopulation()
 
-            if (city.health < city.getMaxHealth()) {
+            if (city.health < city.getMaxHealth() || civHasSignificantlyWeakerMilitaryThanEnemies) {
                 Automation.tryTrainMilitaryUnit(city) // need defenses if city is under attack
                 if (city.cityConstructions.constructionQueue.isNotEmpty())
                     continue // found a unit to build so move on
