@@ -36,6 +36,11 @@ class TurnManager(val civInfo: Civilization) {
         if (civInfo.cities.isNotEmpty() && civInfo.gameInfo.ruleset.technologies.isNotEmpty())
             civInfo.tech.updateResearchProgress()
 
+
+        civInfo.cache.updateCivResources() // If you offered a trade last turn, this turn it will have been accepted/declined
+        for (stockpiledResource in civInfo.getCivResourceSupply().filter { it.resource.isStockpiled() })
+            civInfo.resourceStockpiles.add(stockpiledResource.resource.name, stockpiledResource.amount)
+
         civInfo.civConstructions.startTurn()
         civInfo.attacksSinceTurnStart.clear()
         civInfo.updateStatsForNextTurn() // for things that change when turn passes e.g. golden age, city state influence
@@ -70,8 +75,6 @@ class TurnManager(val civInfo: Civilization) {
                 unit.doAction()
         } else civInfo.hasMovedAutomatedUnits = false
 
-        civInfo.cache.updateCivResources() // If you offered a trade last turn, this turn it will have been accepted/declined
-
         for (tradeRequest in civInfo.tradeRequests.toList()) { // remove trade requests where one of the sides can no longer supply
             val offeringCiv = civInfo.gameInfo.getCivilization(tradeRequest.requestingCiv)
             if (offeringCiv.isDefeated() || !TradeEvaluation().isTradeValid(tradeRequest.trade, civInfo, offeringCiv)) {
@@ -288,7 +291,7 @@ class TurnManager(val civInfo: Civilization) {
         updateWinningCiv() // Maybe we did something this turn to win
     }
 
-    private fun updateWinningCiv(){
+    fun updateWinningCiv(){
         if (civInfo.gameInfo.victoryData != null) return // Game already won
 
         val victoryType = civInfo.victoryManager.getVictoryTypeAchieved()
