@@ -113,7 +113,7 @@ class WorldScreen(
     private val zoomController = ZoomButtonPair(mapHolder)
     internal val minimapWrapper = MinimapHolder(mapHolder)
     private val bottomTileInfoTable = TileInfoTable(viewingCiv)
-    private val notificationsScroll = NotificationsScroll(this)
+    internal val notificationsScroll = NotificationsScroll(this)
     internal val nextTurnButton = NextTurnButton()
     private val statusButtons = StatusButtons(nextTurnButton)
     private val tutorialTaskTable = Table().apply {
@@ -173,6 +173,7 @@ class WorldScreen(
                 }
 
         mapHolder.isAutoScrollEnabled = Gdx.app.type == Application.ApplicationType.Desktop && game.settings.mapAutoScroll
+        mapHolder.mapPanningSpeed = game.settings.mapPanningSpeed
 
         // Don't select unit and change selectedCiv when centering as spectator
         if (viewingCiv.isSpectator())
@@ -301,7 +302,7 @@ class WorldScreen(
         } catch (ex: Throwable) {
             launchOnGLThread {
                 val (message) = LoadGameScreen.getLoadExceptionMessage(ex, "Couldn't download the latest game state!")
-                loadingGamePopup.innerTable.clear()
+                loadingGamePopup.clear()
                 loadingGamePopup.addGoodSizedLabel(message).colspan(2).row()
                 loadingGamePopup.addButton("Retry") {
                     launchOnThreadPool("Load latest multiplayer state after error") {
@@ -320,7 +321,7 @@ class WorldScreen(
     // and we don't get any silly concurrency problems!
     private fun update() {
 
-        if(uiEnabled){
+        if (uiEnabled) {
             displayTutorialsOnUpdate()
 
             bottomUnitTable.update()
@@ -376,7 +377,7 @@ class WorldScreen(
             when {
                 viewingCiv.shouldShowDiplomaticVotingResults() ->
                     UncivGame.Current.pushScreen(DiplomaticVoteResultScreen(gameInfo.diplomaticVictoryVotesCast, viewingCiv))
-                !gameInfo.oneMoreTurnMode && (viewingCiv.isDefeated() || gameInfo.civilizations.any { it.victoryManager.hasWon() }) ->
+                !gameInfo.oneMoreTurnMode && (viewingCiv.isDefeated() || gameInfo.checkForVictory()) ->
                     game.pushScreen(VictoryScreen(this))
                 viewingCiv.greatPeople.freeGreatPeople > 0 ->
                     game.pushScreen(GreatPersonPickerScreen(viewingCiv))
@@ -396,10 +397,10 @@ class WorldScreen(
 
         updateGameplayButtons()
 
-        val maxNotificationsHeight = statusButtons.y -
-                (if (game.settings.showMinimap) minimapWrapper.height else 0f) - bottomTileInfoTable.height - 5f
-        notificationsScroll.update(viewingCiv.notifications, maxNotificationsHeight)
-        notificationsScroll.setTopRight(stage.width - 10f, statusButtons.y - 5f)
+        val coveredNotificationsTop = stage.height - statusButtons.y
+        val coveredNotificationsBottom = bottomTileInfoTable.height +
+                (if (game.settings.showMinimap) minimapWrapper.height else 0f)
+        notificationsScroll.update(viewingCiv.notifications, coveredNotificationsTop, coveredNotificationsBottom)
 
         val posZoomFromRight = if (game.settings.showMinimap) minimapWrapper.width
         else bottomTileInfoTable.width
@@ -634,7 +635,14 @@ class WorldScreen(
 
         updateMultiplayerStatusButton()
 
+        statusButtons.wrap(false)
         statusButtons.pack()
+        val maxWidth = stage.width - techPolicyAndDiplomacy.width - 25f
+        if(statusButtons.width > maxWidth) {
+            statusButtons.width = maxWidth
+            statusButtons.wrap()
+            statusButtons.pack()
+        }
         statusButtons.setPosition(stage.width - statusButtons.width - 10f, topBar.y - statusButtons.height - 10f)
     }
 
